# Documentation

This folder contains examples and documentation for the Deep Java Library (DJL) project.

## [JavaDoc API Reference](https://javadoc.djl.ai/)

Note: when searching in JavaDoc, if your access is denied, please try removing the string `undefined` in the url.

## [Demos](https://djl.ai/website/demo.html)

## Cheat sheet

- [How to load a model](load_model.md)
- [How to collect metrics](how_to_collect_metrics.md)
- [How to use a dataset](development/how_to_use_dataset.md)
- [How to set log level](development/configure_logging.md)
- [Dependency Management](development/dependency_management.md)
- [Cache Management](development/cache_management.md)
- [Memory Management](development/memory_management.md)
- [Troubleshooting](development/troubleshooting.md)
- [Inference Optimization](development/inference_performance_optimization.md)

## [Jupyter notebook tutorials](../jupyter/README.md)

- **[Beginner Jupyter Tutorial](../jupyter/tutorial/README.md)**
- [Run object detection with model zoo](../jupyter/object_detection_with_model_zoo.ipynb)
- [Load pre-trained PyTorch model](../jupyter/load_pytorch_model.ipynb)
- [Load pre-trained Apache MXNet model](../jupyter/load_mxnet_model.ipynb)
- [Transfer learning example](../jupyter/transfer_learning_on_cifar10.ipynb)
- [Question answering example](../jupyter/BERTQA.ipynb)

## [API Examples](../examples/README.md)

- [Single-shot Object Detection example](../examples/docs/object_detection.md)
- [Train your first model](../examples/docs/train_mnist_mlp.md)
- [Image classification example](../examples/docs/image_classification.md)
- [Transfer learning example](../examples/docs/train_cifar10_resnet.md)
- [Train SSD model example](../examples/docs/train_pikachu_ssd.md)
- [Bert question and answer example](../examples/docs/BERT_question_and_answer.md)
- [Instance segmentation example](../examples/docs/instance_segmentation.md)
- [Pose estimation example](../examples/docs/pose_estimation.md)
- [Action recognition example](../examples/docs/action_recognition.md)
- [Sentiment Analysis example](../examples/docs/sentiment_analysis.md)

## Engine Documentation

### [Apache MXNet](../engines/mxnet/README.md)

- [Apache MXNet Gluon Importing](mxnet/how_to_convert_your_model_to_symbol.md) - Apache MXNet Gluon Importing

### [PyTorch](../engines/pytorch/README.md)

- [PyTorch Importing](pytorch/how_to_convert_your_model_to_torchscript.md) - Import TorchScript model

### [TensorFlow](../engines/tensorflow/README.md)

- [TensorFlow Importing](tensorflow/how_to_import_tensorflow_models_in_DJL.md)

### [ONNX Runtime](../engines/onnxruntime/onnxruntime-engine/README.md)

- [Hybrid Engine Operations](hybrid_engine.md) - Using an alternative full Engine for supplemental API support

## Extensions and utilities

- [Android support](../android/README.md)
- [AWS S3 support](../extensions/aws-ai/README.md)
- [Hadoop support](../extensions/hadoop/README.md)

## [Contributor Documentation](development/README.md)

- [Setup development environment](development/setup.md)
- [Development guideline](development/development_guideline.md)

## [Roadmap](roadmap.md)

## [FAQ](faq.md)
